/******************************* Copyright notice *******************************
Copyright (C)2002 by Andrei Loskutov <loskutov@gmx.de>. All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
********************************************************************************/

package adiag.connection;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import adiag.model.Version;

/**
 * The "offline" connection to Perforce-Prozess, through the "old" log file
 * @author Andrei Loskutov
 */
public class FileLogConnector extends Connector {

    /** Instance of Reader for log file */
	private BufferedReader in;
    
    /**
     * Construktor 
     * @param app current GUI, for error feedback etc
     */
    public FileLogConnector(Configuration app) {
        super(app);
    }
    
    /**
     * close connector and all probably opened streams
     * @throws IOException if something goes wrong
     */
    public void close() throws IOException {
        if(in != null){
        	in.close();
        }
    }
          
    /**
     * read native P4 Log 
     * @param apath initial paths for requested files/directories
     * @throws Exception if something goes wrong
     */
    protected void readP4Out(String [] paths) throws Exception {        
        if(paths == null || paths.length ==0){
            return;
        }
		FileReader f = null;
		
        f = new FileReader(getLogFile());

		if (f == null){
			return;
		}
        BufferedReader in = new BufferedReader(f);       

        LineReader in2 = new FileLogLineReader(in);
        
        // we are offline, also we can throw away all old shit
  		init(); 
        
        // we cannot use here "real" paths parameter, because we haven't 
        // "real" P4 instance, also we must
        // get one not-"real" path from the first log line!
        String apath = in2.readLine();

		// strip away conments
		if(apath!=null && apath.startsWith("info:")){
		    out.println(apath);
    		apath = apath.substring(apath.indexOf(":") + 1).trim(); 			
  		} else {
	    	if(apath != null && apath.startsWith("error:")){
            	Version v = getVersion(paths[0], 0, true);
            	v.addDescriptionLine(apath);
            	v.setReceived(true);            	
        	}
  		}   
        parseP4FileLog(apath, in2);
    }
    
}

    /**
     * Wrapper for the BufferedReader
     * @author Andy the Great
     */
    class FileLogLineReader implements LineReader {
		BufferedReader reader;
        /**
         * Constructor.
         */
        public FileLogLineReader(BufferedReader reader) {
            super();
            this.reader = reader;
        }

        /** (non-Javadoc)
         * @see LineReader#readLine()
         */
        public String readLine() throws IOException {
            if(reader != null){
                return reader.readLine();
            } else {
                return null;
            }
        }

        /** (non-Javadoc)
         * @see LineReader#close()
         */
        public void close() throws IOException {
			if(reader != null){
                reader.close();
            }
        }

    }